#include <stdio.h>
#include <stdlib.h>

#include "tokenizer.h"


int main()
{
	
	char * 	msg_contents 	= NULL; 	// string read from the user
	int 	msg_length 	= 0;		// size of string read from user
	int 	retval;				// return value from the getline function
	
	while (1){
		
		printf("Enter a sentence to tokenize: "); 
		
		/*
		using getline (Linux/GNU specific) to read an entire line from the user
	
		getline will do the following for you; 
		(1) 	allocate a buffer just big enough to hold the string you typed. This buffer will be 
			terminated by '\0' and contain the '\n' representing ENTER if applicable.
		(2) 	update the char* which address you used as 1st parameter to make it point to 
			that buffer. 
		(3) 	It will also update the integer which address you used as 2nd
			parameter so that it reflects the size of hte buffer. 
		
		type "man getline" in your konqueror konsole to view the manual page on this very handy function
		*/ 
		
		retval = getline( &msg_contents, &msg_length , stdin);
				
		if (retval == EOF){ 	printf("\n"); return 0; }
		
		// Let's allocate a dynamical array of pointers on char large enough 
		// to hold MAX_NB_TOKENS pointers. We assume we won't have more tokens 
		// than that in any user input and ignore any tokens after this
		char** ptr = tokens_allocate(MAX_NB_TOKENS); 
		
		// we call our tokenize function to work on the user input (msg_contents of size msg_length)
		// and extract each separate tokens which will be copied in a newly allocated strings which 
		// address will be stored inside the dynamical array of pointers we just allocated (ptr of size MAX_NB_TOKENS)
		tokenize(ptr, MAX_NB_TOKENS, msg_contents, strlen(msg_contents)); 
		
		
		// we display all the tokens which have been created and stored inside ptr
		// this will be most likely way less than MAX_NB_TOKENS
		tokens_display(ptr, MAX_NB_TOKENS); 
		
		// We need to deallocate the input buffer, in a production system
		// we would probably avoid to re-allocate memory at each iteration.
		// In our case, getline is so much more practical to use that we just do.
		free (msg_contents); 
		msg_contents = NULL; 
		msg_length = 0;
		
		// we then deallocate each token pointed to by the pointers in ptr and then 
		// deallocate ptr itself
		tokens_deallocate(ptr, MAX_NB_TOKENS); 
		ptr = NULL; 
		
	}
	return 0; 
}
